
# q-expansion of newform 8700.2.g.d, downloaded from the LMFDB on 01 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8700
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[4351, [1, 0]], [5801, [1, 0]], [4177, [-1, 0]], [901, [1, 0]]]
aps_data = [[0, 0], [0, 1], [0, 0], [0, 1], [-3, 0], [0, -1], [0, -3], [4, 0], [0, 0], [1, 0], [-4, 0], [0, 4], [-6, 0], [0, -10], [0, 9], [0, 12], [0, 0], [2, 0], [0, -5], [6, 0], [0, -16], [4, 0], [0, -12], [9, 0], [0, 4], [9, 0], [0, -4], [0, -6], [-11, 0], [0, 3], [0, -2], [21, 0], [0, 6], [-5, 0], [6, 0], [8, 0], [0, 4], [0, -16], [0, 12], [0, 24], [6, 0], [17, 0], [0, 0], [0, 14], [0, -6], [13, 0], [14, 0], [0, -19], [0, 0], [22, 0], [0, 6], [6, 0], [11, 0], [-3, 0], [0, -18], [0, 0], [-21, 0], [14, 0], [0, 19], [24, 0], [0, -4], [0, 21], [0, 16], [-15, 0], [0, -1], [0, 15], [-10, 0], [0, 22], [0, 24], [-14, 0], [0, -6], [0, 0], [0, 4], [0, -10], [-26, 0], [0, 24], [-15, 0], [0, -38], [24, 0], [-32, 0], [6, 0], [20, 0], [12, 0], [0, -16], [1, 0], [0, -27], [-9, 0], [0, 1], [18, 0], [0, -7], [0, -12], [24, 0], [0, -8], [12, 0], [-5, 0], [0, -33], [24, 0], [18, 0], [0, 23], [38, 0], [0, 25], [0, 12], [0, -15], [-15, 0], [-16, 0], [0, 16], [0, 24], [0, 0], [-33, 0], [-46, 0], [0, 16], [0, 11], [0, 6], [46, 0], [11, 0], [27, 0], [0, -31], [0, -18], [0, 45], [-15, 0], [29, 0], [0, -13], [0, 21], [0, -24], [35, 0], [-18, 0], [34, 0], [42, 0], [0, -44], [0, -22], [-2, 0], [0, -9], [20, 0], [0, 16], [-48, 0], [-26, 0], [0, -6], [0, 28], [0, -18], [39, 0], [47, 0], [-18, 0], [0, -34], [0, 48], [40, 0], [15, 0], [0, -34], [0, 18], [4, 0], [0, -42], [0, 22], [-21, 0], [0, -28], [0, 39], [0, -8], [-39, 0], [-11, 0], [54, 0], [0, 31], [-6, 0], [0, -3], [0, -6], [0, -44], [12, 0], [0, 24], [0, 24], [-7, 0], [0, -8], [1, 0], [0, -39], [0, 0], [5, 0], [-48, 0], [0, -4], [-32, 0], [18, 0], [-1, 0], [-3, 0], [0, -22], [-35, 0], [0, -38], [-39, 0], [0, -13], [0, -48], [0, 18], [30, 0], [0, -2], [0, -46], [10, 0], [-60, 0], [0, 41], [0, -9], [32, 0], [27, 0], [0, -33], [0, -18], [56, 0], [0, 23], [0, -12], [0, 6], [-30, 0], [5, 0], [0, -2], [4, 0], [39, 0], [0, -12], [-50, 0], [0, 60], [30, 0], [8, 0], [0, -56], [-36, 0], [0, -52], [0, 3], [-15, 0], [-67, 0], [0, -59], [-33, 0], [0, -6], [0, -27], [-22, 0], [-41, 0], [-9, 0], [0, 2], [0, -6], [34, 0], [0, -6], [-12, 0], [0, 16], [0, 0], [0, 26], [-56, 0], [14, 0], [-18, 0], [0, -49], [0, 39], [34, 0], [0, 54], [-48, 0], [-75, 0], [0, -33], [20, 0], [0, 23], [16, 0], [0, 48], [-60, 0], [0, -32], [-42, 0], [31, 0], [0, 15], [0, 4], [48, 0], [0, -3], [16, 0], [0, 42], [-78, 0], [44, 0], [0, -62], [0, -6], [0, -47], [0, -64], [0, -33], [55, 0], [0, -46], [0, -18], [-26, 0], [27, 0], [-18, 0], [0, 32], [0, -36], [-7, 0], [0, -11], [0, -10], [-26, 0], [0, 40], [0, -34], [0, 36], [1, 0], [56, 0], [0, 0], [0, -36], [23, 0], [0, -6], [23, 0], [0, 22], [45, 0], [0, -64], [0, -3], [73, 0], [48, 0], [54, 0], [0, -6], [0, 24], [-48, 0], [0, -10], [0, 0], [-58, 0], [0, -84], [18, 0], [0, -56], [0, 8], [0, -30], [-38, 0], [0, 60], [-52, 0], [0, 70], [0, -27], [-38, 0], [-72, 0], [0, 41], [0, 48], [21, 0], [6, 0], [0, 35], [0, -42], [-65, 0], [-60, 0], [24, 0], [0, -19], [57, 0], [-16, 0], [0, -35], [-30, 0], [0, 2], [0, -6], [32, 0], [52, 0], [0, 8], [0, 51], [0, 54], [-88, 0], [0, 6], [88, 0], [0, 69], [-88, 0], [0, 18], [-23, 0], [0, 9], [-58, 0], [0, -29], [0, 50], [0, -12], [15, 0], [29, 0], [0, 12], [60, 0], [-70, 0], [0, 64], [-33, 0], [0, -57], [-4, 0], [0, -35], [18, 0], [0, -64], [-2, 0], [0, -54], [-15, 0], [60, 0], [0, -39], [0, -24], [0, 43], [0, 0], [0, -75], [-66, 0], [0, 10], [0, -52], [0, -33], [0, 77], [-64, 0], [96, 0], [55, 0], [0, -66], [-51, 0], [65, 0], [0, 94], [-21, 0], [-51, 0], [0, 56], [-42, 0], [0, 94], [66, 0], [0, -72], [0, 22], [0, -33], [100, 0], [0, 24], [8, 0], [0, -47], [0, 50], [0, 51], [10, 0], [0, -54], [-45, 0], [0, 88], [-39, 0], [0, 65], [-68, 0], [-102, 0], [5, 0], [-51, 0], [-14, 0], [0, -54], [0, 76], [0, -72], [18, 0], [20, 0], [0, 55], [-45, 0], [0, -28], [-54, 0], [0, -1], [0, 72], [0, 6], [-76, 0], [0, -74], [57, 0], [15, 0], [0, 7], [0, -57], [0, -12], [-30, 0], [0, 16], [0, -48], [12, 0], [0, 29], [0, 54], [0, -6], [-33, 0], [-97, 0], [96, 0], [32, 0], [84, 0], [76, 0], [0, -54], [0, 82], [-42, 0], [97, 0], [101, 0], [0, -23], [-17, 0], [0, 90], [-21, 0], [-77, 0], [54, 0], [26, 0], [0, 60], [0, -58], [0, 108], [-8, 0], [50, 0], [0, -32], [54, 0], [0, 30], [81, 0], [0, 106], [3, 0], [-44, 0], [27, 0], [0, 62], [0, -36], [70, 0], [23, 0], [-60, 0], [62, 0], [0, 43], [0, -19], [-11, 0], [0, 3], [24, 0], [92, 0], [0, -94], [0, -36], [36, 0], [20, 0], [-24, 0], [0, 98], [-48, 0], [-10, 0], [0, -48], [0, -36], [0, -10], [-54, 0], [0, 7], [105, 0], [0, 26], [0, -30], [88, 0], [-33, 0], [-116, 0], [8, 0], [0, 88], [0, 3], [-41, 0], [0, 66], [-36, 0], [-22, 0], [0, 1], [0, -42], [58, 0], [-13, 0], [39, 0], [0, 71], [0, 81]]
